module Aginx
  module Generators
    class ControllerGenerator < Rails::Generators::NamedBase
      class_option :columns, type: :array, default: []
      class_option :skip_model, type: :boolean, default: false,
        desc: "Indicates when to skip generate columns from an named model, this will be true automatically when arguments is not empty"
      class_option :skip_search, type: :boolean, default: false,
        desc: "Indicates when to skip generate codes about search" # TODO --skip-search开关用于控制器是否支持搜索
      class_option :skip_sort, type: :boolean, default: false,
        desc: "Indicates when to skip generate codes about sort"   # TODO --skip-sort开关用于控制器是否支持排序

      source_root File.expand_path('../templates', __FILE__)

      def init
        prepare
      end

      def create_controller
        template 'controller.rb', File.join('app/controllers', class_path, "#{@model_table_name}_controller.rb")
      end

      def create_route
        routes_code = <<-EOF
        resources :#{@model_table_name} do
          collection do
            delete 'multi_destroy'
          end
        end
                EOF

        # 兼容 agi_scaffold, 索引字符串前添加空格
        inject_into_file 'config/routes.rb', before: "        # Do not delete this line for agi_scaffold" do
          routes_code
        end
      end

      private

      def prepare
        check_namespace
        set_model
      end

      def check_namespace
      raise 'Namespace is not specified!' if class_path.length == 0
      end

      def set_model
        @model_table_name = file_name.underscore.pluralize
        @model_name = file_name.camelize.singularize
        @model_columns = options[:columns]
        if @model_columns.length == 0 && !options[:skip_model]
          model = @model_name.constantize
          if model.respond_to? :columns
            @model_columns = model.columns.map{|column| "#{column.name}:#{column.type}"}
          else
            raise NameError
          end
        end
      rescue NameError
        raise "Can't any table information about #{@model_name} model"
      end

    end
  end
end
